"""This module is for the translation mapping data of the
following document:

Document file: set_debug_mode.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# set_debug_mode interface':
    '',

    'This page explains the `set_debug_mode` function interface.':
    '',

    '## What interface is this?':
    '',

    'The `set_debug_mode` function interface sets the debug mode setting. This setting appends the debug information (Python function or method calls and arguments information) to the exported HTML.':  # noqa
    '',

    '## Notes':
    '',

    'The debug mode setting appends lots of information to the HTML. As a result, the exporting time becomes long, and the HTML file size becomes large.\n\nAlso, this setting ignores the `minify` setting.':  # noqa
    '',

    '## Basic usage':
    '',

    'After the stage instantiation, you can set the debug mode by the `set_debug_mode` function. This function requires the stage instance since the stage instance resets the settings at first, including the debug mode.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nap.set_debug_mode()\nsprite: ap.Sprite = ap.Sprite()\nint_1: ap.Int = ap.Int(10)\n\nap.save_overall_html(\n    dest_dir_path=\'set_debug_mode_basic_usage/\')\n```':  # noqa
    '',

    'This setting appends the information (Python\'s function and method callings, its module and class names, and argument information) as the JavaScript comment to the exported HTML, like the following:':  # noqa
    '',

    '```js\n...\n  //////////////////////////////////////////////////////////////////////\n  // [__init__ 1] started.\n  // module name: apysc._display.sprite\n  // class: Sprite\n  // arguments and variables:\n  //    variable_name = None\n  //    self = Sprite(\'\')()\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] started.\n    // module name: apysc._type.array\n    // class: Array\n    // arguments and variables:\n    //    value = []\n    //    self = []()\n      //////////////////////////////////////////////////////////////////////\n      // [_append_constructor_expression 2] started.\n      // module name: apysc._type.array\n      // class: Array\n      // arguments and variables:\n      //    self = [](arr_2)\n        var arr_2 = [];\n      // [_append_constructor_expression 2] ended.\n      // module name: apysc._type.array\n      // class: Array\n      //////////////////////////////////////////////////////////////////////\n    // [__init__ 2] ended.\n    // module name: apysc._type.array\n    // class: Array\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 1] started.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    // arguments and variables:\n    //    variable_name = \'sp_1\'\n    //    self = Sprite(\'\')()\n    // [__init__ 1] ended.\n    // module name: apysc._display.display_object\n    // class: DisplayObject\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [_append_constructor_expression 1] started.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    // arguments and variables:\n    //    self = Sprite(\'sp_1\')(sp_1)\n      var sp_1 = stage.nested();\n    // [_append_constructor_expression 1] ended.\n    // module name: apysc._display.sprite\n    // class: Sprite\n    //////////////////////////////////////////////////////////////////////\n...\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [Unset debug mode interface](unset_debug_mode.md)':
    '',

    '## set_debug_mode API':
    '',

    '<!-- Docstring: apysc._html.debug_mode.set_debug_mode -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `set_debug_mode() -> None`<hr>\n\n**[Interface summary]** Set the debug mode for the HTML and JavaScript debugging. If calling this function, this interface applies the following setting: <br> ・Disabling HTML minify setting. <br> ・Changing to append per each interface JavaScript divider string.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> ap.set_debug_mode()\n>>> int_val: ap.Int = ap.Int(10)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: quick_start.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Quick start guide':
    '',

    'This page explains the first step of the apysc library journey.':
    '',

    '## Installing':
    '',

    'To use apysc library Python 3.6 or the later version is required.\n\nYou can use the pip command to install apysc.':  # noqa
    '',

    '```\n$ pip install apysc\n```':
    '',

    '':
    '',

    '## Create stage and export HTML':
    '',

    '`Stage` instance is apysc\'s space for displaying each graphics. You can set arguments of `stage_width` for width setting, `stage_height` for height setting, and `background_color` for background.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage = ap.Stage(stage_width=300, stage_height=180, background_color=\'#333\')\n```':  # noqa
    '',

    'Then you can export each HTML and js file by the `save_overall_html` function (in this case, that code displays only the black background stage).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=300, stage_height=180, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nap.save_overall_html(\n    dest_dir_path=\'quick_start_stage_creation/\')\n```':  # noqa
    '',

    'This code will create each HTML and js files to `dest_dir_path`. You can confirm an exported result by opening `index.html` (`quick_start_stage_creation/index.html`), as follows:\n\n<iframe src="static/quick_start_stage_creation/index.html" width="300" height="180"></iframe>':  # noqa
    '',

    '## Add sprite container and vector graphics':
    '',

    'The `Sprite` class is the container object of each display object, and it can make vector graphics with the `graphics` property.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw polyline vector graphics.\nsprite.graphics.line_style(color=\'#fff\', thickness=3)\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=100, y=50)\nsprite.graphics.line_to(x=50, y=100)\nsprite.graphics.line_to(x=100, y=100)\n\n# Draw rectangle vector graphic.\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'quick_start_sprite_graphics/\')\n```':  # noqa
    '',

    '<iframe src="static/quick_start_sprite_graphics/index.html" width="250" height="150"></iframe>\n\nPlease see each interface documentation page for more details of `Sprite` and `Graphics`\\.':  # noqa
    '',

    '## See also':
    '',

    '- [Sprite class](sprite.md)\n- [Draw interfaces abstract](draw_interfaces_abstract.md)':  # noqa
    '',

}



"""This module is for the event handler's expression
scope interfaces implementations.
"""

from typing import Any
from typing import List
from typing import Optional
from typing import Tuple

from apysc._type.variable_name_interface import VariableNameInterface


class HandlerScope:
    """
    Class for a handler scope. The apysc uses this class at
    a with-statement.
    """

    _handler_name: str
    _instance: VariableNameInterface

    def __init__(
            self, *, handler_name: str,
            instance: VariableNameInterface) -> None:
        """
        Class for a handler scope. The apysc uses this class at
        a with-statement.

        Parameters
        ----------
        handler_name : str
            Target handler's name.
        instance : VariableNameInterface
            Instance will be binded the target handler.
        """
        self._handler_name = handler_name
        self._instance = instance

    def __enter__(self) -> None:
        """
        Enter and set an event handler scope setting.
        """
        _increment_scope_count()
        _save_handler_calling_stack(
            handler_name=self._handler_name, instance=self._instance)

    def __exit__(self, *args: Any) -> None:
        """
        Exit and remove an event handler scope setting.

        Parameters
        ----------
        *args : list
            Positional arguments.
        """
        _delete_handler_calling_stack(handler_name=self._handler_name)
        _decrement_scope_count()


def _save_handler_calling_stack(
        *, handler_name: str, instance: VariableNameInterface) -> None:
    """
    Save the handler calling stack data to the SQLite.

    Parameters
    ----------
    handler_name : str
        Target handler's name.
    instance : VariableNameInterface
        Instance will be binded the target handler.
    """
    from apysc._expression import expression_data_util
    scope_count: int = get_current_event_handler_scope_count()
    variable_name: str = instance.variable_name
    query: str = (
        'INSERT INTO '
        f'{expression_data_util.TableName.HANDLER_CALLING_STACK.value}'
        '(handler_name, scope_count, variable_name) '
        f"VALUES('{handler_name}', {scope_count}, '{variable_name}');"
    )
    expression_data_util.exec_query(sql=query)


def remove_suffix_num_from_handler_name(*, handler_name: str) -> str:
    """
    Remove the suffix number from a specified handler name.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    handler_name : str
        Result handler's name.
    """
    splitted: List[str] = handler_name.split('_')
    splitted = splitted[:-1]
    handler_name = '_'.join(splitted)
    return handler_name


def _delete_handler_calling_stack(*, handler_name: str) -> None:
    """
    Delete the handler calling stack data from the SQLite.

    Parameters
    ----------
    handler_name : str
        Target handler's name.
    """
    from apysc._expression import expression_data_util
    scope_count: int = get_current_event_handler_scope_count()
    query: str = (
        'DELETE FROM '
        f'{expression_data_util.TableName.HANDLER_CALLING_STACK.value} '
        f"WHERE handler_name = '{handler_name}' "
        f'AND scope_count = {scope_count};'
    )
    expression_data_util.exec_query(sql=query)


class TemporaryNotHandlerScope:
    """
    Class temporarily sets up a scope that is not
    a handler. The apysc uses this at a with-statement.
    """

    _original_scope_count: int

    def __init__(self) -> None:
        """
        Class temporarily sets up a scope that is not a handler.
        The apysc uses this at a with-statement.
        """
        self._original_scope_count = get_current_event_handler_scope_count()

    def __enter__(self) -> None:
        """
        Enter and set the scope count to zero.
        """
        _save_current_scope_count(count=0)

    def __exit__(self, *args: Any) -> None:
        """
        Exit and revert the scope count.

        Parameters
        ----------
        *args : list
            Positional arguments.
        """
        _save_current_scope_count(count=self._original_scope_count)


def _increment_scope_count() -> None:
    """
    Increment current scope count.
    """
    scope_count: int = get_current_event_handler_scope_count()
    scope_count += 1
    _save_current_scope_count(count=scope_count)


def _decrement_scope_count() -> None:
    """
    Decrement current scope count.
    """
    scope_count: int = get_current_event_handler_scope_count()
    scope_count -= 1
    scope_count = max(scope_count, 0)
    _save_current_scope_count(count=scope_count)


def _save_current_scope_count(*, count: int) -> None:
    """
    Save current scope count.

    Parameters
    ----------
    count : int
        Scope count to save.
    """
    from apysc._expression import expression_data_util
    query: str = (
        'DELETE FROM '
        f'{expression_data_util.TableName.EVENT_HANDLER_SCOPE_COUNT.value};'
    )
    expression_data_util.exec_query(sql=query, commit=False)
    query = (
        'INSERT INTO '
        f'{expression_data_util.TableName.EVENT_HANDLER_SCOPE_COUNT.value}'
        f'(count) VALUES({count});'
    )
    expression_data_util.exec_query(sql=query)


def get_current_event_handler_scope_count() -> int:
    """
    Get a current event handler's scope count.

    Returns
    -------
    scope_count : int
        Current event handler's scope count. If normal
        handler's call, then this interface returns 1,
        or call the other handler in handler's function,
        then this interface returns 2 or more count.
    """
    from apysc._expression import expression_data_util
    query: str = (
        'SELECT count FROM '
        f'{expression_data_util.TableName.EVENT_HANDLER_SCOPE_COUNT.value} '
        'LIMIT 1;'
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple] = expression_data_util.cursor.fetchone()
    if result is None:
        return 0
    scope_count: int = int(result[0])
    return scope_count



"""This module is for the translation mapping data of the
following document:

Document file: animation_move.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# animation_move interface':
    '',

    'This page explains the `animation_move` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `animation_move` method interface creates an `AnimationMove` instance. You can animate the x and y coordinates with it.\n\nThis interface exists on a `DisplayObject` subclass instance, such as the `Sprite` or `Rectangle`.':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example sets the coordinates animation (from x=50, y=50 to x=100, y=100) with the `animation_move` method.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=50, y=50, duration=1000)\n    animation_move.animation_complete(on_animation_complete_2)\n    animation_move.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=100, y=100, duration=1000)\n    animation_move.animation_complete(on_animation_complete_1)\n    animation_move.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(\n    x=100, y=100, duration=1000)\nanimation_move.animation_complete(on_animation_complete_1)\nanimation_move.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_move_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_move_basic_usage/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## animation_move API':
    '',

    '<!-- Docstring: apysc._animation.animation_move_interface.AnimationMoveInterface.animation_move -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_move(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int], *, duration:Union[int, apysc._type.int.Int]=3000, delay:Union[int, apysc._type.int.Int]=0, easing:apysc._animation.easing.Easing=<Easing.LINEAR: \'function(x) {return x;}\'>) -> apysc._animation.animation_move.AnimationMove`<hr>\n\n**[Interface summary]** Set the x and y coordinates animation settings.<hr>\n\n**[Parameters]**\n\n- `x`: Int or int\n  - Destination of the x-coordinate.\n- `y`: Int or int\n  - Destination of the y-coordinate.\n- `duration`: Int or int, default 3000\n  - Milliseconds before an animation ends.\n- `delay`: Int or int, default 0\n  - Milliseconds before an animation starts.\n- `easing`: Easing, default Easing.LINEAR\n  - Easing setting.\n\n<hr>\n\n**[Returns]**\n\n- `animation_move`: AnimationMove\n  - Created animation setting instance.\n\n<hr>\n\n**[Notes]**\n\nTo start this animation, you need to call the `start` method of the returned instance.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=1)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_move(\n...     x=100, y=150,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Animation interfaces duration setting document](https://simon-ritchie.github.io/apysc/animation_duration.html)\n- [Animation interfaces delay setting document](https://simon-ritchie.github.io/apysc/animation_delay.html)\n- [Each animation interface return value document](https://simon-ritchie.github.io/apysc/animation_return_value.html)\n- [Sequential animation setting document](https://simon-ritchie.github.io/apysc/sequential_animation.html)\n- [animation_parallel interface document](https://simon-ritchie.github.io/apysc/animation_parallel.html)\n- [Easing enum document](https://simon-ritchie.github.io/apysc/easing_enum.html)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_sort.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class sort interface':
    '',

    'This page explains the `Array` class `sort` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `sort` method interface sorts an array\'s values (ascending order).':  # noqa
    '',

    '## Basic usage':
    '',

    'The `sort` method requires no arguments and sorts values in place (no return value).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([5, 1, 3])\narr.sort()\nassert arr == [1, 3, 5]\n```':  # noqa
    '',

    '':
    '',

    '## Sort values by descending order':
    '',

    'If you need to sort values by descending order, then use the `reverse` method also:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 3, 2])\narr.sort()\narr.reverse()\nassert arr == [3, 2, 1]\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [Array class reverse interface](array_reverse.md)':
    '',

    '## sort API':
    '',

    '<!-- Docstring: apysc._type.array.Array.sort -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `sort(self, *, ascending:bool=True) -> None`<hr>\n\n**[Interface summary]** Sort this array in place.<hr>\n\n**[Parameters]**\n\n- `ascending`: bool, default True\n  - Sort by ascending or not. If False is specified, this interface sorts values descending.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([3, 5, 1, 4, 2])\n>>> arr.sort()\n>>> arr\nArray([1, 2, 3, 4, 5])\n\n>>> arr.sort(ascending=False)\n>>> arr\nArray([5, 4, 3, 2, 1])\n```':  # noqa
    '',

}



"""Base class implementation for that has JavaScript dictionary
data structure.
"""


class DictionaryStructure:
    """
    Base class implementation for that has JavaScript
    dictionary data structure.

    The apysc treats an instance that inherits this base
    class as dictionary value in JavaScript (e.g., assert_equal
    dictionary type branch instruction).
    """



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_polygon.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics draw_polygon interface':
    '',

    'This page explains the `Graphics` class `draw_polygon` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `draw_polygon` interface draws vector polygon graphics. This interface works slightly similar to the `line_to` and `move_to` interfaces, but the paths do not need to be closed.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `draw_polygon` interface has the `points` argument, which determines the polygon vertices coordinates.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_polygon(\n    points=ap.Array([\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ]))\n\n# Draw the diamond shape with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array([\n        ap.Point2D(x=175, y=50),\n        ap.Point2D(x=150, y=75),\n        ap.Point2D(x=175, y=100),\n        ap.Point2D(x=200, y=75),\n    ]))\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_polygon_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_polygon_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## Difference between the line_to and draw_polygon interfaces':
    '',

    'If you set the fill color, the `draw_polygon` interface becomes slightly similar to the `line_to` (and `move_to`) interfaces. So, for example, the following codes both draw the triangle.\n\nThe `draw_polygon` interface draws the left rectangle. Similarly, the `move_to` and `line_to` interfaces draw the right one.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array([\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ]))\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_polygon_line_to_difference_1/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_polygon_line_to_difference_1/index.html" width="250" height="150"></iframe>\n\nBut there is a difference in whether closing the paths is necessary or not. This difference becomes significant when you set the line style setting. The `line_to` interface does not close the paths from end coordinates to start coordinates.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n# Set the line style to see the difference.\nsprite.graphics.line_style(color=\'#fff\', thickness=3)\n\n# Draw the triangle with the draw_polygon interface.\nsprite.graphics.draw_polygon(\n    points=ap.Array([\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=100),\n        ap.Point2D(x=100, y=100),\n    ]))\n\n# Draw the triangle with the move_to and line_to interfaces.\nsprite.graphics.move_to(x=175, y=50)\nsprite.graphics.line_to(x=150, y=100)\nsprite.graphics.line_to(x=200, y=100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_polygon_line_to_difference_2/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_polygon_line_to_difference_2/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## Return value':
    '',

    'The `draw_polygon` interface returns the `Polygon` instance. And that has the basic interface as same as the other type graphics instances. The `Polygon` instance also has the `append_line_point` method interface to append points dynamically.\n\nFor instance, the following code appends the point and changes from the triangle to the rectangle.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n# Draw the triangle.\npolygon: ap.Polygon = sprite.graphics.draw_polygon(\n    points=ap.Array([\n        ap.Point2D(x=75, y=50),\n        ap.Point2D(x=50, y=75),\n        ap.Point2D(x=75, y=100),\n    ]))\n\n# Append the point and change to the rectangle dynamically.\npolygon.append_line_point(x=100, y=75)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_polygon_append_line_point/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_polygon_append_line_point/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## draw_polygon API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.draw_polygon -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `draw_polygon(self, points:Union[List[apysc._geom.point2d.Point2D], apysc._type.array.Array[apysc._geom.point2d.Point2D]]) -> \'_polyg.Polygon\'`<hr>\n\n**[Interface summary]** Draw a polygon vector graphic. This interface is similar to the Polyline class (created by `move_to` or `line_to`). But unlike that, this interface connects the last point and the start point.<hr>\n\n**[Parameters]**\n\n- `points`: list of Point2D or Array.\n  - Polygon vertex points.\n\n<hr>\n\n**[Returns]**\n\n- `polygon`: Polygon\n  - Created polygon graphics instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> polygon: ap.Polygon = sprite.graphics.draw_polygon(\n...     points=[\n...         ap.Point2D(x=25, y=0),\n...         ap.Point2D(x=0, y=50),\n...         ap.Point2D(x=50, y=50),\n...     ])\n>>> polygon.fill_color\nString(\'#00aaff\')\n```':  # noqa
    '',

}



"""This module is for the path data fundamental
utility implementations.
"""

from typing import List

from apysc._geom.path_data_base import PathDataBase
from apysc._html.debug_mode import add_debug_info_setting


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def make_paths_expression_from_list(
        *, path_data_list: List[PathDataBase]) -> str:
    """
    Make a paths expression from a specified list of path data.

    Parameters
    ----------
    path_data_list : list of PathDataBase
        Target path data settings, such as the ap.PathData.MoveTo.

    Returns
    -------
    expression : str
        Created expression string.
    """
    expression: str = ''
    for path_data in path_data_list:
        if expression != '':
            expression += ' + " " + '
        expression += path_data._get_svg_str()
    return expression



"""This module is for the definitions of the custom
event types.
"""

from enum import Enum


class CustomEventType(Enum):

    TIMER_COMPLETE = 'timer_complete'
    ANIMATION_COMPLETE = 'animation_complete'



"""This module is for the translation mapping data of the
following document:

Document file: graphics_line_style.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics line_style interface':
    '',

    'This page explains the `Graphics` class `line_style` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `line_style` interface sets each line style, such as the line color, line alpha, line thickness, line dot setting. This interface maintains these settings until it is called again or called the `clear` method (similar to the `begin_fill` interface).':  # noqa
    '',

    '## Basic usage':
    '',

    'Draw vector graphics interfaces (e.g., the `draw_rect` or `line_to`) use these line settings when creating. Therefore, calling the `line_style` method is necessary before calling each drawing interface.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=162,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw a white line with 3px line thickness.\nsprite.graphics.line_style(color=\'#ccc\', thickness=8)\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=150, y=50)\n\n# Line style setting will be maintained.\nsprite.graphics.move_to(x=50, y=80)\nsprite.graphics.line_to(x=150, y=80)\n\n# Change line color and thickness.\nsprite.graphics.line_style(color=\'#0af\', thickness=3)\nsprite.graphics.move_to(x=50, y=110)\nsprite.graphics.line_to(x=150, y=110)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_basics/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_basics/index.html" width="200" height="162"></iframe>':  # noqa
    '',

    '## Line-color setting':
    '',

    'The required `color` argument sets the line color.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=102,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set a cyan line color and draw the line.\nsprite.graphics.line_style(color=\'#0af\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=50, y_end=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_color/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_color/index.html" width="200" height="102"></iframe>\n\nIf you want to clear line color, specify a blank string to this argument.\n\nFor example, the result line graphic becomes invisible since the following code clears the line color setting.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=102,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set a cyan line color.\nsprite.graphics.line_style(color=\'#0af\', thickness=4)\n\n# Clear the line color by specifying a blank string.\nsprite.graphics.line_style(color=\'\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=50, y_end=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_clear_line_color/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_clear_line_color/index.html" width="200" height="102"></iframe>\n\nColor code is acceptable like the following list (same as `begin_fill` interface `color` argument):\n\n- Six characters, e.g., `#00aaff`.\n- Three characters, e.g., `#0af` (this becomes `#00aaff`).\n- Single character, e.g., `#5` (this becomes `#000005`).\n- Skipped `#` symbol, e.g., `0af` (this becomes `#00aaff`).\n- Blank string, e.g., `\'\'` (this clears line color setting).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=162,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# The six characters line color setting (a cyan color).\nsprite.graphics.line_style(color=\'#00aaff\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=50, y_end=50)\n\n# The three characters line color setting (a magenta color).\nsprite.graphics.line_style(color=\'#f0a\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=80, y_end=80)\n\n# The one character line color setting (a black color).\nsprite.graphics.line_style(color=\'#5\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=110, y_end=110)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_color_color_code/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_color_color_code/index.html" width="200" height="162"></iframe>':  # noqa
    '',

    '## Line thickness setting':
    '',

    'The `thickness` argument sets the line thickness. It can accept greater than or equal to 1.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=165,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 1-pixel line thickness.\nsprite.graphics.line_style(color=\'#0af\', thickness=1)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=50, y_end=50)\n\n# Set 4-pixel line thickness.\nsprite.graphics.line_style(color=\'#0af\', thickness=4)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=80, y_end=80)\n\n# Set 10-pixel line thickness.\nsprite.graphics.line_style(color=\'#0af\', thickness=10)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=110, y_end=110)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_thickness/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_thickness/index.html" width="200" height="165"></iframe>':  # noqa
    '',

    '## Line alpha (opacity) setting':
    '',

    'A line alpha (opacity) can be set by the `alpha` argument. It can accept 0.0 (transparent) to 1.0 (opaque).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Draw the cyan line from upper-left to lower-right.\nsprite.graphics.line_style(color=\'#0af\', thickness=15, alpha=0.3)\nsprite.graphics.draw_line(x_start=50, x_end=100, y_start=50, y_end=100)\n\n# Draw the magenta line from upper-right to lower-left.\nsprite.graphics.line_style(color=\'#f0a\', thickness=15, alpha=0.3)\nsprite.graphics.draw_line(x_start=100, x_end=50, y_start=50, y_end=100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_alpha/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_alpha/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Line cap setting':
    '',

    'Line cap setting changes line edge style. The `cap` argument sets this style setting, and `LineCaps` enum values are acceptable.\n\nThere are three `LineCaps` options, as follows:\n\n- BUTT: This is the default value, and it sets no cap.\n- ROUND: This changes the line edge to the rounded one.\n- SQUARE: This is similar to BUTT, but it increases the line length by the squared edge.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=180,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# BUTT caps setting (default).\nsprite.graphics.line_style(color=\'#0af\', thickness=20, cap=ap.LineCaps.BUTT)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=50, y_end=50)\n\n# ROUND caps setting.\nsprite.graphics.line_style(color=\'#0af\', thickness=20, cap=ap.LineCaps.ROUND)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=90, y_end=90)\n\n# SQUARE caps setting (same line length setting as BUTT line,\n# but this will be longer for the caps).\nsprite.graphics.line_style(color=\'#0af\', thickness=20, cap=ap.LineCaps.SQUARE)\nsprite.graphics.draw_line(x_start=50, x_end=150, y_start=130, y_end=130)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_caps/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_caps/index.html" width="200" height="180"></iframe>':  # noqa
    '',

    '## Line joints setting':
    '',

    'Line joints setting changes the line vertices style. The `joints` argument sets this style, and `LineJoints` enum values are acceptable. The `Polyline` class (`move_to` and `line_to` interfaces) mainly uses this argument.\n\nThere are three LineJoints enum values, as follows:\n\n- MITER: This setting sets the style like a picture frame vertices. This setting is the default style setting.\n- ROUND: This setting sets the rounded vertices style.\n- BEVEL: This setting sets a sloping vertices style.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=350,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set MITER joints setting and draw the polyline.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=10, joints=ap.LineJoints.MITER)\nsprite.graphics.move_to(x=50, y=100)\nsprite.graphics.line_to(x=75, y=50)\nsprite.graphics.line_to(x=100, y=100)\n\n# Set ROUND joints setting and draw the polyline.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=10, joints=ap.LineJoints.ROUND)\nsprite.graphics.move_to(x=150, y=100)\nsprite.graphics.line_to(x=175, y=50)\nsprite.graphics.line_to(x=200, y=100)\n\n# Set BEVEL joints setting and draw the polyline.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=10, joints=ap.LineJoints.BEVEL)\nsprite.graphics.move_to(x=250, y=100)\nsprite.graphics.line_to(x=275, y=50)\nsprite.graphics.line_to(x=300, y=100)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_joints/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_joints/index.html" width="350" height="150"></iframe>':  # noqa
    '',

    '## Line dot setting':
    '',

    'Line dot setting changes the line to dotted line. The `dot_setting` argument (`LineDotSetting` value) sets this setting. It can change dot size by the `dot_size` argument (greater than or equal to 1 value is acceptable).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=300,\n    stage_height=160,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the line dot settings with 2-pixel dot size and draw the dotted line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5, dot_setting=ap.LineDotSetting(dot_size=2))\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=250, y=50)\n\n# Set the line dot settings with 5-pixel dot size and draw the dotted line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5, dot_setting=ap.LineDotSetting(dot_size=5))\nsprite.graphics.move_to(x=50, y=80)\nsprite.graphics.line_to(x=250, y=80)\n\n# Set the line dot settings with 10-pixel dot size and draw the dotted line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5, dot_setting=ap.LineDotSetting(dot_size=10))\nsprite.graphics.move_to(x=50, y=110)\nsprite.graphics.line_to(x=250, y=110)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_dot_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_dot_setting/index.html" width="300" height="160"></iframe>\n\nThis setting (or the other similar settings) also changes the `Rectangle` or other graphics classes.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the line dot setting with 2-pixel dot size and draw the rectangle.\n# Fill color setting is skipped.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5, dot_setting=ap.LineDotSetting(dot_size=2))\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Draw the rectangle with the dotted line setting and the fill color.\nsprite.graphics.begin_fill(color=\'#038\')\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_dot_setting_rectangle/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_dot_setting_rectangle/index.html" width="250" height="150"></iframe>\n\nNotes: This setting will be ignored by `draw_line`, `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`, and `draw_dash_dotted_line` interfaces.':  # noqa
    '',

    '## Line dash setting':
    '',

    'Line dash setting changes the line to the dashed line. The `dash_setting` argument (`LineDashSetting` value) sets this setting. It can change dash size and space size by the `dash_size` and `space_size` arguments.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=300,\n    stage_height=130,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 10-pixel dash size and 3-pixel space size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=3,\n    dash_setting=ap.LineDashSetting(dash_size=10, space_size=3))\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=250, y=50)\n\n# Set 15-pixel dash size and 5-pixel space size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=3,\n    dash_setting=ap.LineDashSetting(dash_size=15, space_size=5))\nsprite.graphics.move_to(x=50, y=80)\nsprite.graphics.line_to(x=250, y=80)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_dash_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_dash_setting/index.html" width="300" height="130"></iframe>\n\nNotes: This setting will be ignored by `draw_line`, `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`, and `draw_dash_dotted_line` interfaces.':  # noqa
    '',

    '## Line round dot setting':
    '',

    'Line round dot setting changes the line to the round dotted line. The `round_dot_setting` argument (`LineRoundDotSetting` value) sets this setting. It can change round size and space size by the `round_size` and `space_size` arguments.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=300,\n    stage_height=130,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel round size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5,\n    round_dot_setting=ap.LineRoundDotSetting(round_size=5, space_size=5))\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=250, y=50)\n\n# Set 10-pixel round size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=5,\n    round_dot_setting=ap.LineRoundDotSetting(round_size=10, space_size=5))\nsprite.graphics.move_to(x=50, y=80)\nsprite.graphics.line_to(x=250, y=80)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_round_dot_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_round_dot_setting/index.html" width="300" height="130"></iframe>\n\nNotes: Since this setting uses the `cap` setting internally, this setting ignores the `cap` setting, increasing the line length by the capsize.\n\nNotes: This setting will be ignored by `draw_line`, `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`, and `draw_dash_dotted_line` interfaces.':  # noqa
    '',

    '## Line dash-dot setting':
    '',

    'Line dash-dot setting changes the line to the dash-dotted line (also called long dashed short dashed line or one-dot chain line). The `dash_dot_setting` arguments set this setting. This argument accepts the `dot_size` (short dashed size), `dash_size` (long dashed size), and `space_size` arguments.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=300,\n    stage_height=130,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 3-pixel dot size and 10-pixel dash size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=3,\n    dash_dot_setting=ap.LineDashDotSetting(\n        dot_size=3, dash_size=10, space_size=3))\nsprite.graphics.move_to(x=50, y=50)\nsprite.graphics.line_to(x=250, y=50)\n\n# Set 5-pixel dot size and 15-pixel dash size and draw the line.\nsprite.graphics.line_style(\n    color=\'#0af\', thickness=3,\n    dash_dot_setting=ap.LineDashDotSetting(\n        dot_size=5, dash_size=15, space_size=3))\nsprite.graphics.move_to(x=50, y=80)\nsprite.graphics.line_to(x=250, y=80)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_line_style_line_dash_dot_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_line_style_line_dash_dot_setting/index.html" width="300" height="130"></iframe>\n\nNotes: This setting will be ignored by `draw_line`, `draw_dotted_line`, `draw_dashed_line`, `draw_round_dotted_line`, and `draw_dash_dotted_line` interfaces.':  # noqa
    '',

    '## line_style API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_style -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `line_style(self, color:~StrOrString, *, thickness:Union[int, apysc._type.int.Int]=1, alpha:Union[float, apysc._type.number.Number]=1.0, cap:Union[apysc._display.line_caps.LineCaps, NoneType]=None, joints:Union[apysc._display.line_joints.LineJoints, NoneType]=None, dot_setting:Union[apysc._display.line_dot_setting.LineDotSetting, NoneType]=None, dash_setting:Union[apysc._display.line_dash_setting.LineDashSetting, NoneType]=None, round_dot_setting:Union[apysc._display.line_round_dot_setting.LineRoundDotSetting, NoneType]=None, dash_dot_setting:Union[apysc._display.line_dash_dot_setting.LineDashDotSetting, NoneType]=None) -> None`<hr>\n\n**[Interface summary]** Set line style values.<hr>\n\n**[Parameters]**\n\n- `color`: String or str\n  - Hexadecimal color string. e.g., \'#00aaff\'\n- `thickness`: Int or int, default 1\n  - Line thickness (minimum value is 1).\n- `alpha`: float or Number, default 1.0\n  - Line color opacity (0.0 to 1.0).\n- `cap`: LineCaps or None, default None\n  - Line cap (edge style) setting. The not line-related graphics (e.g., Rectangle ignores this, conversely used by Polyline) ignore this setting.\n- `joints`: LineJoints or None, default None\n  - Line vertices (joints) style setting. The not polyline-related graphics (e.g., Rectangle ignores this, conversely used by Polyline) ignore this setting.\n- `dot_setting`: LineDotSetting or None, default None\n  - Dash setting. If this is specified, it makes a line dashed.\n- `dash_setting`: LineDashSetting or None, default None\n  - Dash setting. If this is specified, it makes a line dashed.\n- `round_dot_setting`: LineRoundDotSetting or None, default None\n  - Round dot setting. If this is specified, it makes a line round dotted. Notes: since this style uses a cap setting, it overrides cap and line thickness settings. And it increases the amount of line size. If you want to adjust to the same width of a normal line when using move_to and line_to interfaces, add half-round size to start x-coordinate and subtract from end e-coordinate. e.g., `this.move_to(x + round_size / 2, y)`, `this.line_to(x - round_size / 2, y)`\n- `dash_dot_setting`: LineDashDotSetting or None, default None\n  - Dash dot (1-dot chain) setting. If this is specified, it makes a line 1-dot chained.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5, alpha=0.5,\n...     cap=ap.LineCaps.ROUND)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50)\n>>> line.line_color\nString(\'#ffffff\')\n\n>>> line.line_thickness\nInt(5)\n\n>>> line.line_alpha\nNumber(0.5)\n\n>>> line.line_cap\nString(\'round\')\n```':  # noqa
    '',

    '':
    '',

    '## line_color property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_color -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line color.<hr>\n\n**[Returns]**\n\n- `line_color`: String\n  - Current line color (hexadecimal string, e.g., \'#00aaff\'). If not be set, this interface returns a blank string.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5, alpha=0.5,\n...     cap=ap.LineCaps.ROUND)\n>>> sprite.graphics.line_color\nString(\'#ffffff\')\n```':  # noqa
    '',

    '':
    '',

    '## line_thickness property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_thickness -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line thickness.<hr>\n\n**[Returns]**\n\n- `line_thickness`: Int\n  - Current line thickness.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5, alpha=0.5)\n>>> sprite.graphics.line_thickness\nInt(5)\n```':  # noqa
    '',

    '':
    '',

    '## line_alpha property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_alpha -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line color opacity.<hr>\n\n**[Returns]**\n\n- `line_alpha`: Number\n  - Current line opacity (0.0 to 1.0).\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5, alpha=0.5,\n...     cap=ap.LineCaps.ROUND)\n>>> sprite.graphics.line_alpha\nNumber(0.5)\n```':  # noqa
    '',

    '':
    '',

    '## line_cap property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_cap -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line cap (edge) style setting.<hr>\n\n**[Returns]**\n\n- `line_cap`: String\n  - Current line cap (edge) style setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5, alpha=0.5,\n...     cap=ap.LineCaps.ROUND)\n>>> sprite.graphics.line_cap\nString(\'round\')\n```':  # noqa
    '',

    '':
    '',

    '## line_joints property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_joints -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line joints (vertices) style setting.<hr>\n\n**[Returns]**\n\n- `line_joints`: String\n  - Current line joints (vertices) style setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5,\n...     joints=ap.LineJoints.ROUND)\n>>> sprite.graphics.line_joints\nString(\'round\')\n```':  # noqa
    '',

    '':
    '',

    '## line_dot_setting property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_dot_setting -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line dot setting.<hr>\n\n**[Returns]**\n\n- `line_dot_setting`: LineDotSetting or None\n  - Current line dot setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5,\n...     dot_setting=ap.LineDotSetting(dot_size=5))\n>>> sprite.graphics.line_dot_setting.dot_size\nInt(5)\n```':  # noqa
    '',

    '':
    '',

    '## line_dash_setting property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_dash_setting -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line dash setting.<hr>\n\n**[Returns]**\n\n- `line_dash_setting`: LineDashSetting or None\n  - Current line dash setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5,\n...     dash_setting=ap.LineDashSetting(\n...         dash_size=10, space_size=5))\n>>> sprite.graphics.line_dash_setting.dash_size\nInt(10)\n\n>>> sprite.graphics.line_dash_setting.space_size\nInt(5)\n```':  # noqa
    '',

    '':
    '',

    '## line_round_dot_setting property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_round_dot_setting -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line round dot setting.<hr>\n\n**[Returns]**\n\n- `line_round_dot_setting`: LineRoundDotSetting or None\n  - Current line round dot setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5,\n...     round_dot_setting=ap.LineRoundDotSetting(\n...         round_size=6, space_size=3))\n>>> sprite.graphics.line_round_dot_setting.round_size\nInt(6)\n\n>>> sprite.graphics.line_round_dot_setting.space_size\nInt(3)\n```':  # noqa
    '',

    '':
    '',

    '## line_dash_dot_setting property API':
    '',

    '<!-- Docstring: apysc._display.line_style_interface.LineStyleInterface.line_dash_dot_setting -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get current line dash dot setting.<hr>\n\n**[Returns]**\n\n- `line_dash_dot_setting`: LineDashDotSetting or None\n  - Current line dash dot setting.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(\n...     color=\'#fff\', thickness=5,\n...     dash_dot_setting=ap.LineDashDotSetting(\n...         dot_size=2, dash_size=5, space_size=3))\n>>> sprite.graphics.line_dash_dot_setting.dot_size\nInt(2)\n\n>>> sprite.graphics.line_dash_dot_setting.dash_size\nInt(5)\n\n>>> sprite.graphics.line_dash_dot_setting.space_size\nInt(3)\n```':  # noqa
    '',

}



"""Class implementation for the line round-dot setting
interface.
"""

from typing import Dict
from typing import Optional

from apysc._display.line_cap_interface import LineCapInterface
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_thickness_interface import LineThicknessInterface
from apysc._html.debug_mode import add_debug_info_setting


class LineRoundDotSettingInterface(LineCapInterface, LineThicknessInterface):

    _line_round_dot_setting: Optional[LineRoundDotSetting]

    def _initialize_line_round_dot_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_round_dot_setting if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_line_round_dot_setting'):
            return
        self._line_round_dot_setting = None

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineRoundDotSettingInterface')
    def line_round_dot_setting(self) -> Optional[LineRoundDotSetting]:
        """
        Get this instance's line round dot setting.

        Returns
        -------
        line_round_dot_setting : LineRoundDotSetting or None
            Line round dot setting.

        References
        ----------
        - Graphics line_round_dot interface document
            - https://simon-ritchie.github.io/apysc/graphics_line_round_dot_setting.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50)
        >>> line.line_round_dot_setting = ap.LineRoundDotSetting(
        ...     round_size=10, space_size=5)
        >>> line.line_round_dot_setting.round_size
        Int(10)

        >>> line.line_round_dot_setting.space_size
        Int(5)
        """
        self._initialize_line_round_dot_setting_if_not_initialized()
        return self._line_round_dot_setting

    @line_round_dot_setting.setter
    def line_round_dot_setting(
            self, value: Optional[LineRoundDotSetting]) -> None:
        """
        Set line round dot setting.

        Notes
        -----
        This property updating will affect line cap and
        thickness settings.

        Parameters
        ----------
        value : LineRoundDotSetting or None
            Line round setting to set.
        """
        from apysc._html.debug_mode import DebugInfo
        with DebugInfo(
                callable_='line_round_dot_setting', args=[value], kwargs={},
                module_name=__name__,
                class_name=LineRoundDotSettingInterface.__name__):
            import apysc as ap
            from apysc._validation import display_validation
            self._update_line_round_dot_setting_and_skip_appending_exp(
                value=value)
            if value is not None:
                self.line_cap = ap.LineCaps.ROUND
                self.line_thickness = value.round_size
            else:
                self.line_cap = ap.LineCaps.BUTT
            self._append_line_round_dot_setting_update_expression()
            display_validation.validate_multiple_line_settings_isnt_set(
                any_instance=self)

    def _update_line_round_dot_setting_and_skip_appending_exp(
            self, *, value: Optional[LineRoundDotSetting]) -> None:
        """
        Update line round setting and skip appending expression.

        Parameters
        ----------
        value : LineRoundSetting or None
            Line round-dot settings to set.
        """
        if value is not None and not isinstance(value, LineRoundDotSetting):
            raise TypeError(
                'Not supported line_round_dot_setting type specified: '
                f'{type(value)}'
                '\nAcceptable ones are: LineRoundDotSetting or None.')
        self._line_round_dot_setting = value

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineRoundDotSettingInterface')
    def _append_line_round_dot_setting_update_expression(self) -> None:
        """
        Append line round dot setting updating expression.
        """
        import apysc as ap
        if self._line_round_dot_setting is None:
            setting_str: str = '""'
        else:
            round_size_name: str = \
                self._line_round_dot_setting.round_size.variable_name
            space_size_name: str = \
                self._line_round_dot_setting.space_size.variable_name
            setting_str = (
                f'"1 " + String({round_size_name} + {space_size_name})'
            )
        expression: str = (
            f'{self.variable_name}.css("stroke-dasharray", {setting_str});'
        )
        ap.append_js_expression(expression=expression)

    _line_round_dot_setting_snapshots: Dict[
        str, Optional[LineRoundDotSetting]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_round_dot_setting_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_round_dot_setting_snapshots',
            value=self._line_round_dot_setting, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._line_round_dot_setting = self._line_round_dot_setting_snapshots[
            snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_skew.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# GraphicsBase skew_x and skew_y interfaces':
    '',

    'This page explains the `GraphicsBase` class (base class of each graphic, such as the `Rectangle`) `skew_x` and `skew_y` property interfaces.\n\nEach interface value type is the `Int` value.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `skew_x` property skews an object\'s x-axis. Conversely, the `skew_y` property skew a y-axis. These interfaces have getter and setter interfaces.\n\nThe following example shows you the default rectangle (left) and the skewed 50px in the x-direction rectangle (right).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\nright_rectangle.skew_x = ap.Int(50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_base_skew_x_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_base_skew_x_basic_usage/index.html" width="250" height="150"></iframe>\n\nThe following example skews the rectangle in the y-direction incrementally.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.skew_y += 1\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _RectOptions = {\'rectangle\': rectangle}\ntimer: ap.Timer = ap.Timer(\n    handler=on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_base_skew_y_incremental_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_base_skew_y_incremental_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## skew_x property API':
    '',

    '<!-- Docstring: apysc._display.skew_x_interface.SkewXInterface.skew_x -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a current skew x value of the instance.<hr>\n\n**[Returns]**\n\n- `skew_x`: Int\n  - Current skew x value of this instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.skew_x = ap.Int(50)\n>>> rectangle.skew_x\nInt(50)\n```':  # noqa
    '',

    '':
    '',

    '## skew_y property API':
    '',

    '<!-- Docstring: apysc._display.skew_y_interface.SkewYInterface.skew_y -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a current skew y value of the instance.<hr>\n\n**[Returns]**\n\n- `skew_y`: Int\n  - Current skew y value of the instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.skew_y = ap.Int(50)\n>>> rectangle.skew_y\nInt(50)\n```':  # noqa
    '',

}



"""Class implementation for the CSS interface.
"""

from typing import Dict
from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_interface import RevertInterface
from apysc._type.string import String
from apysc._type.variable_name_interface import VariableNameInterface


class CssInterface(VariableNameInterface, RevertInterface):

    _css: Dict[str, String]

    def _initialize_css_if_not_initialized(self) -> None:
        """
        Initialize the _css attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_css'):
            return
        self._css = {}

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CssInterface')
    def get_css(self, name: Union[str, String]) -> String:
        """
        Get a CSS value string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').

        Returns
        -------
        css : ap.String
            CSS value.

        References
        ----------
        - Display object get_css and set_css interfaces document
            - https://simon-ritchie.github.io/apysc/display_object_get_and_set_css.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af', alpha=0.5)
        >>> sprite.set_css(name='display', value='none')
        >>> sprite.get_css(name='display')
        String('none')
        """
        import apysc as ap
        from apysc._converter import to_builtin_val_from_apysc
        self._initialize_css_if_not_initialized()
        name_: str = to_builtin_val_from_apysc.\
            get_builtin_str_from_apysc_val(string=name)
        if name_ in self._css:
            css: ap.String = self._css[name_]._copy()
        else:
            css = ap.String('')
        self._append_get_css_expresion(name=name, css=css)
        return css

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CssInterface')
    def _append_get_css_expresion(
            self, *, name: Union[str, String], css: String) -> None:
        """
        Append a CSS getter expression string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        css : String
            CSS value.
        """
        import apysc as ap
        from apysc._type import value_util
        name_value_str: str = value_util.get_value_str_for_expression(
            value=name)
        css_value_str: str = value_util.get_value_str_for_expression(
            value=css)
        expression: str = (
            f'{css_value_str} = {self.variable_name}.'
            f'css({name_value_str});'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CssInterface')
    def set_css(
            self, name: Union[str, String],
            value: Union[str, String]) -> None:
        """
        Set a specified value string to the CSS.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        value : str or String
            A CSS value string (e.g., 'none').

        References
        ----------
        - Display object get_css and set_css interfaces document
            - https://simon-ritchie.github.io/apysc/display_object_get_and_set_css.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af', alpha=0.5)
        >>> sprite.set_css(name='display', value='none')
        >>> sprite.get_css(name='display')
        String('none')
        """
        import apysc as ap
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._converter import to_builtin_val_from_apysc
        self._initialize_css_if_not_initialized()
        name_: str = to_builtin_val_from_apysc.\
            get_builtin_str_from_apysc_val(string=name)
        value_: ap.String = to_apysc_val_from_builtin.\
            get_copied_string_from_builtin_val(string=value)
        self._css[name_] = value_
        self._append_set_css_expression(name=name, value=value)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CssInterface')
    def _append_set_css_expression(
            self, *, name: Union[str, String],
            value: Union[str, String]) -> None:
        """
        Append a CSS setter expression string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        value : str or String
            A CSS value string (e.g., 'none').
        """
        import apysc as ap
        from apysc._type import value_util
        name_value_str: str = value_util.get_value_str_for_expression(
            value=name)
        value_str: str = value_util.get_value_str_for_expression(
            value=value)
        expression: str = (
            f'{self.variable_name}.css({name_value_str}, {value_str});'
        )
        ap.append_js_expression(expression=expression)

    _css_snapshot: Dict[str, Dict[str, String]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_css_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_css_snapshot',
            value={**self._css}, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._css = self._css_snapshot[snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: why_apysc_doesnt_use_python_builtin_data_type.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Why the apysc library doesn\'t use the Python built-in data type':
    '',

    'This page explains why the apysc library doesn\'t use the Python built-in data type, such as `int`\\, `float`\\, `bool`\\, `list`\\. And why the apysc library uses the apysc data type like the `Int`\\, `Number`\\, `Array` instead.':  # noqa
    '',

    '## apysc needs to convert Python to JavaScript and track variables change':  # noqa
    '',

    'The apysc library needs to track variable creation and update to convert the Python code to JavaScript. For this reason, apysc using the original data types, such as `Int`\\, `Number` (`Float`)\\, `String`\\, `Boolean`\\, `Array`\\, and `Dictionary`\\.\n\nOccasionally, these are unnecessary to create HTML. Still, these types become essential when you use the asynchronous function, such as the event handler.\n\nThe apysc library automatically sets each variable\'s names and uses them when exporting the HTML and JavaScript files. It also tracks variables creation and updating and applies them to the exported JavaScript.\n\nUsing the Python built-in data type, these variables\' values become fixed (the apysc doesn\'t apply asynchronous function\'s changes).':  # noqa
    '',

}



"""Class implementation for the parent-related interfaces.
"""

from typing import Any
from typing import Dict
from typing import Optional

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_interface import RevertInterface


class ParentInterface(RevertInterface):

    _parent: Optional[Any] = None

    @property
    def parent(self) -> Optional[Any]:
        """
        Get parent instance that has a add_child and remove_child
        interfaces.

        Returns
        -------
        parent : any parent instance or None
            Parent instance with `add_child` and `remove_child`
            interfaces. If this instance does not have a parent
            instance (not added child), this interface returns None.

        References
        ----------
        - Display object parent interfaces document
            - https://simon-ritchie.github.io/apysc/display_object_parent.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite_1: ap.Sprite = ap.Sprite()
        >>> sprite_1.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite_1.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> sprite_2: ap.Sprite = ap.Sprite()
        >>> sprite_2.add_child(rectangle)
        >>> rectangle.parent == sprite_2
        True
        """
        return self._parent

    @parent.setter
    def parent(self, value: Optional[Any]) -> None:
        """
        Set parent instance.

        Notes
        -----
        Only use this setter method at add_child or that's related
        interface.

        Parameters
        ----------
        value : *
            Parent instance to be set.

        Raises
        ------
        ValueError
            If specified instance is not None and hasn't `ChildInterface`
            interfaces.

        References
        ----------
        - Display object parent interfaces document
            - https://simon-ritchie.github.io/apysc/display_object_parent.html  # noqa
        """
        from apysc._validation import parent_validation
        parent_validation.validate_parent_instance(parent=value)
        parent_validation.validate_parent_contains_child(
            parent=value, child=self)
        self._parent = value

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='ParentInterface')
    def remove_from_parent(self) -> None:
        """
        Remove this instance from a parent.

        Raises
        ------
        ValueError
            If a parent is None (there is no parent).
        """
        from apysc._display.child_interface import ChildInterface
        from apysc._display.child_interface import \
            append_expression_of_remove_child
        from apysc._display.display_object import DisplayObject
        parent: Optional[ChildInterface] = self._parent
        child: DisplayObject = self  # type: ignore
        if parent is not None:
            parent.remove_child(child=child)
        else:
            append_expression_of_remove_child(child=child)

    _parent_snapshots: Dict[str, Optional[Any]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name='_parent_snapshots',
            value=self._parent, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._parent = self._parent_snapshots[snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: animation_parallel.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# animation_parallel interface':
    '',

    'This page explains the `animation_parallel` interface.':
    '',

    '## What interface is this?':
    '',

    'The `animation_parallel` method interface creates an `AnimationParallel` instance. You can set multiple simultaneous animations to the same instance with it.\n\nThis interface exists on a `DisplayObject` subclass instance, such as the `Sprite` or `Rectangle` class.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can use this interface with the `animation_parallel` method. The `animations` list argument requires any animation settings.\n\nThe following example sets the simultaneous animations of the x, fill alpha, fill color, and line thickness.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=400, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.line_style(color=\'#fff\', thickness=3)\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300),\n        rectangle.animation_fill_color(fill_color=\'#f0a\'),\n        rectangle.animation_fill_alpha(alpha=0.3),\n        rectangle.animation_line_thickness(thickness=7),\n    ],\n    duration=3000, delay=3000, easing=ap.Easing.EASE_OUT_QUINT,\n).start()\n\nap.save_overall_html(\n    dest_dir_path=\'animation_parallel_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_parallel_basic_usage/index.html" width="400" height="150"></iframe>':  # noqa
    '',

    '## Note for each animation\'s duration, delay, and easing setting':
    '',

    'Animation settings of the `duration`\\, `delay`\\, and `easing` in the `animations` argument can\'t be changed since these animation settings are referring to the `animation_parallel` arguments. If you set these parameters in the `animations` arguments, setting raise an error:':  # noqa
    '',

    '```py\n...\nrectangle.animation_parallel(\n    animations=[\n        rectangle.animation_x(x=300, duration=1000),\n    ],\n    duration=3000, delay=2000, easing=ap.Easing.EASE_OUT_QUINT,\n)\n...\n```':  # noqa
    '',

    '':
    '',

    '```\nValueError: There is an animation target that is changed duration setting: 1000\nThe duration setting of animation in the `animations` argument can not be changed.\nTarget animation type: <class \'apysc._animation.animation_x.AnimationX\'>\n```':  # noqa
    '',

    '':
    '',

    '## animation_parallel API':
    '',

    '<!-- Docstring: apysc._animation.animation_parallel_interface.AnimationParallelInterface.animation_parallel -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_parallel(self, animations:List[apysc._animation.animation_base.AnimationBase], *, duration:Union[int, apysc._type.int.Int]=3000, delay:Union[int, apysc._type.int.Int]=0, easing:apysc._animation.easing.Easing=<Easing.LINEAR: \'function(x) {return x;}\'>) -> apysc._animation.animation_parallel.AnimationParallel`<hr>\n\n**[Interface summary]** Set the parallel animation setting.<hr>\n\n**[Parameters]**\n\n- `animations`: list of AnimationBase\n  - Target animation settings.\n- `duration`: Int or int, default 3000\n  - Milliseconds before an animation ends.\n- `delay`: Int or int, default 0\n  - Milliseconds before an animation starts.\n- `easing`: Easing, default Easing.LINEAR\n  - Easing setting.\n\n<hr>\n\n**[Returns]**\n\n- `animation_parallel`: AnimationParallel\n  - Created animation setting instance.\n\n<hr>\n\n**[Raises]**\n\n- ValueError: <br> ・If the animations\' target is not this instance. <br> ・If there are changed duration, delay, or easing animation settings in the `animations` list.\n\n<hr>\n\n**[Notes]**\n\n ・To start this animation, you need to call the `start` method of the returned instance. <br> ・The `animations` argument can\'t contains the `AnimationParallel` instance. <br> ・This interface ignores the duration, delay, and easing arguments in the `animations` argument (this interface uses self-arguments instead).<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_parallel(\n...     animations=[\n...         rectangle.animation_x(x=100),\n...         rectangle.animation_fill_color(fill_color=\'#f0a\'),\n...         rectangle.animation_fill_alpha(alpha=0.5),\n...     ],\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Animation interfaces duration setting document](https://simon-ritchie.github.io/apysc/animation_duration.html)\n- [Animation interfaces delay setting document](https://simon-ritchie.github.io/apysc/animation_delay.html)\n- [Each animation interface return value document](https://simon-ritchie.github.io/apysc/animation_return_value.html)\n- [Sequential animation setting document](https://simon-ritchie.github.io/apysc/sequential_animation.html)\n- [Easing enum document](https://simon-ritchie.github.io/apysc/easing_enum.html)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_pop.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class pop interface':
    '',

    'This page explains the `Array` class `pop` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `pop` method interface removes the last value from an array and returns that value.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `pop` method accepts no arguments and returns the last value, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3])\nlast_value: int = arr.pop()\nassert last_value == 3\n```':  # noqa
    '',

    '':
    '',

    '## pop API':
    '',

    '<!-- Docstring: apysc._type.array.Array.pop -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `pop(self) -> ~T`<hr>\n\n**[Interface summary]** Remove this array\'s last value and return it.<hr>\n\n**[Returns]**\n\n- `value`: *\n  - Removed value.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> popped_val: int = arr.pop()\n>>> popped_val\n3\n\n>>> arr\nArray([1, 2])\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_rect.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics draw_rect interface':
    '',

    'This page explains the `Graphics` class `draw_rect` method interface.':
    '',

    '## What interface is this?':
    '',

    '`draw_rect` interface draws vector rectangle graphics.':
    '',

    '## Basic usage':
    '',

    '`draw_rect` interface has `x`, `y`, `width`, and `height` arguments. `x` and `y` are rectangle coordinates setting, and `width` and `height` will determine rectangle size.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_rect_basic_usage/\')\n```':  # noqa
    '',

    'The previous script draws horizontal rectangle graphics.\n\n<iframe src="static/graphics_draw_rect_basic_usage/index.html" width="200" height="150"></iframe>\n\nNotes: `begin_fill` call (fill color setting) is necessary before `draw_rect` interface call. If you skip it, it displays nothing on stage.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.draw_rect(x=50, y=50, width=100, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_rect_basic_usage_skipped_begin_fill/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_rect_basic_usage_skipped_begin_fill/index.html" width="200" height="150"></iframe>':  # noqa
    '',

    '## Rectangle instance':
    '',

    '`draw_rect` interface will return `Rectangle` instance. You can update each setting or bind events.\n\nFor instance, the following script sets the mouse event to `Rectangle` and updates x position in the handler (`on_click`).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Created event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x = ap.Int(100)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_rect_rectangle/\')\n```':  # noqa
    '',

    'If you click rectangle, the handler changes the x position to 100.\n\n<iframe src="static/graphics_draw_rect_rectangle/index.html" width="200" height="150"></iframe>':  # noqa
    '',

    '## draw_rect API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.draw_rect -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `draw_rect(self, x:Union[int, apysc._type.int.Int], y:Union[int, apysc._type.int.Int], width:Union[int, apysc._type.int.Int], height:Union[int, apysc._type.int.Int]) -> apysc._display.rectangle.Rectangle`<hr>\n\n**[Interface summary]** Draw a rectangle vector graphics.<hr>\n\n**[Parameters]**\n\n- `x`: Int or int\n  - X position to start drawing.\n- `y`: Int or int\n  - Y position to start drawing.\n- `width`: Int or int\n  - Rectangle width.\n- `height`: Int or int\n  - Rectangle height.\n\n<hr>\n\n**[Returns]**\n\n- `rectangle`: Rectangle\n  - Created rectangle.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.x\nInt(50)\n\n>>> rectangle.width\nInt(50)\n\n>>> rectangle.fill_color\nString(\'#00aaff\')\n```':  # noqa
    '',

}



"""Class implementation for the animation_reverse interface.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_interface import VariableNameInterface


class AnimationReverseInterface(VariableNameInterface):

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationReverseInterface')
    def animation_reverse(self) -> None:
        """
        Reverse all running animations.

        Notes
        -----
        Suppose you call this interface multiple times and
        animations reach the beginning or end of the animation.
        In that case, this interface ignores the reverse instruction.
        This behavior means that the same interval's timer tick
        reverse setting does not work correctly (since the same
        interval setting reaches the animation start).

        References
        ----------
        - animation_reverse interface document
            - https://simon-ritchie.github.io/apysc/animation_reverse.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        >>> def on_timer(
        ...         e: ap.TimerEvent,
        ...         options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options['rectangle']
        ...     rectangle.animation_reverse()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.animation_x(
        ...     x=100,
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        >>> options: RectOptions = {'rectangle': rectangle}
        >>> ap.Timer(on_timer, delay=750, options=options).start()
        """
        import apysc as ap
        expression: str = (
            f'{self.variable_name}.timeline().reverse();'
        )
        ap.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: assert_true_and_false.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# assert_true and assert_false interfaces':
    '',

    'This page explains the `assert_true` and `assert_false` function interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `assert_true` function interface asserts a specified `Boolean` value is true. Conversely, the `assert_false` function interface asserts a specified `Boolean` value is false.':  # noqa
    '',

    '## See also':
    '',

    '- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)':  # noqa
    '',

    '## Basic usage':
    '',

    'The `assert_true` and `assert_false` interfaces requires `value` argument. The `type_strict` and `msg` arguments are optional (default value of the `type_strict` argument is `True`).\n\nIf the `type_strict` argument is `True`, the assertion will use the JavaScript strict comparison operator (`===`). For instance, if the `value` is `Int(1)` and the `type_strict` is `True`, an assertion will fail (because of the comparison between the `Boolean` and `Int`). Conversely, if the `type_strict` is `False`, `Int(1)` will pass the `assert_true` assertion.\n\nThese interfaces display an assertion result on the browser console.\n\nThe following assertion example (`assert_true` and value is the `Boolean(True)`) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nbool_1: ap.Boolean = ap.Boolean(True)\nap.assert_true(bool_1, msg=\'Boolean value is not True!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_true_basic_usage_1/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: true\n```':  # noqa
    '',

    '<iframe src="static/assert_true_basic_usage_1/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_true` and value is the `Boolean(False)`) fails:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_true(bool_1, msg=\'Boolean value is not True!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_true_basic_usage_2/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_true]\nRight-side variable name: b_3\nLeft value: true right value: false\n...\nAssertion failed: Boolean value is not True!\n```':  # noqa
    '',

    '<iframe src="static/assert_true_basic_usage_2/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_true` and value is the `Int(1)` and `type_strict` is `True`) will fail:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=True, msg=\'Value is not Boolean(True)!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_true_basic_usage_3/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n...\nAssertion failed: Value is not Boolean(True)!\n```':  # noqa
    '',

    '<iframe src="static/assert_true_basic_usage_3/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_true` and value is the `Int(1)` and `type_strict` is `False`) will pass:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nint_1: ap.Int = ap.Int(1)\nap.assert_true(int_1, type_strict=False, msg=\'Value is not True!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_true_basic_usage_4/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_true]\nRight-side variable name: i_11\nLeft value: true right value: 1\n```':  # noqa
    '',

    '<iframe src="static/assert_true_basic_usage_4/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_false` and value is the `Boolean(False)`) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nbool_1: ap.Boolean = ap.Boolean(False)\nap.assert_false(bool_1, msg=\'Value is not False!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_false_basic_usage_1/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_false]\nRight-side variable name: b_3\nLeft value: false right value: false\n```':  # noqa
    '',

    '<iframe src="static/assert_false_basic_usage_1/index.html" width="0" height="0"></iframe>':  # noqa
    '',

    '## assert_true API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_true -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_true(value:Any, *, type_strict:bool=True, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for true condition.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Target value to check.\n- `type_strict`: bool, default True\n  - Whether strictly check actual value or not. For example, if type_strict is True, an integer of 1 fails tests. On the contrary (if type_strict is False), an integer of 1 passes tests.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 10\n>>> ap.assert_true(boolean)\n```':  # noqa
    '',

    '':
    '',

    '## assert_false API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_false -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_false(value:Any, *, type_strict:bool=True, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for false condition.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Target value to check.\n- `type_strict`: bool, default True\n  - Whether strictly check actual value or not. For example, if type_strict is True, an integer of 0 fails tests. On the contrary (if type_strict is False), an integer of 0 passes tests.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> boolean: ap.Boolean = int_val == 11\n>>> ap.assert_false(boolean)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: display_object_mouse_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Display object mouse event binding interfaces':
    '',

    'This page explains the `DisplayObject` class mouse event binding interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'Each `DisplayObject` instance has the mouse event binding interfaces, like the click, mouse over, mouse move.\n\nThese interfaces can bind the mouse event to a `DisplayObject` instance. So, for instance, you can assign any function to handle when a `DisplayObject` instance click.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can bind event handler (callable) with each interface, like the `click`\\, `mouseover`\\.\n\nThe following example binds the click event handler, and if you click the rectangle, the fill color is changed.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'display_object_mouse_event_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/display_object_mouse_event_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    'For more details, please see the following pages:\n\n- [Basic mouse event interfaces](mouse_event_basic.md)\n- [Click interface](click.md)\n- [Mousedown and mouseup interfaces](mousedown_and_mouseup.md)\n- [Mouseover and mouseout interfaces](mouseover_and_mouseout.md)\n- [Mousemove interface](mousemove.md)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: for.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# For':
    '',

    'This page explains the `For` class.\n\nBefore reading on, maybe it is helpful to read the following page (the apysc uses the `For` class for the same reason for each data type):\n\n- [Why the apysc library doesn\'t use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)':  # noqa
    '',

    '## What is the For class?':
    '',

    'The `For` class is the apysc for loop class. It behaves like the Python built-in `for` keyword.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `For` class requires using the `with` statement. The `as` keyword value becomes the `Int` type index (an `i` variable).\n\nThe following example draws the three rectangles in the `with For` block:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\narr: ap.Array[int] = ap.Array(range(3))\ni: ap.Int\nwith ap.For(arr) as i:\n    sprite.graphics.draw_rect(\n        x=i * 100 + 50, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'for_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/for_basic_usage/index.html" width="350" height="150"></iframe>\n\nThe `For` class constructor\'s first argument accepts an `Array` or `Dictionary` value. If you pass a `Dictionary` value, the `as` keyword value becomes a `String` value, not `Int`\\.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\ndict_val: ap.Dictionary = ap.Dictionary(\n    {\'magenta\': ap.String(\'#f0a\'), \'cyan\': ap.String(\'#0af\')})\nkey: ap.String\nwith ap.For(dict_val) as key:\n    color: ap.String = dict_val[key]\n    sprite.graphics.begin_fill(color=color)\n    condition_1: ap.Boolean = key == \'magenta\'\n    condition_2: ap.Boolean = key == \'cyan\'\n    with ap.If(condition_1):\n        sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n    with ap.Elif(condition_2):\n        sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.save_overall_html(\n    dest_dir_path=\'for_basic_usage_with_dict/\')\n```':  # noqa
    '',

    '<iframe src="static/for_basic_usage_with_dict/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Each branch instruction class\'s scope variables reverting setting](branch_instruction_variables_reverting_setting.md)\n  - Notes: the `For` class also has the same arguments and behaves in the same way.':  # noqa
    '',

    '## For API':
    '',

    '<!-- Docstring: apysc._loop._for.For.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, arr_or_dict:Union[apysc._type.array.Array, apysc._type.dictionary.Dictionary], *, locals_:Union[Dict[str, Any], NoneType]=None, globals_:Union[Dict[str, Any], NoneType]=None) -> None`<hr>\n\n**[Interface summary]** A class to append for the (loop) expression.<hr>\n\n**[Parameters]**\n\n- `arr_or_dict`: Array or Dictionary\n  - Array or Dictionary instance to iterate.\n- `locals_`: dict or None, default None\n  - Current scope\'s local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameInterface variables (like Int, Sprite) at the end of a `For` scope. This setting is useful when you don\'t want to update each variable by implementing the `For` scope.\n- `globals_`: dict or None, default None\n  - Current scope\'s global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array(range(3))\n>>> with ap.For(arr) as i:\n...     ap.trace(\'Loop index is:\', i)\n```':  # noqa
    '',

}



"""This module is for the HTML and JavaScript debugging-mode
setting interface implementations.
"""

import functools
import inspect
from typing import Any
from typing import Callable
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type
from typing import TypeVar
from typing import Union

from apysc._expression.indent_num import Indent


def set_debug_mode() -> None:
    """
    Set the debug mode for the HTML and JavaScript debugging.
    If calling this function, this interface applies
    the following setting:
    - Disabling HTML minify setting.
    - Changing to append per each interface JavaScript divider string.

    References
    ----------
    - set_debug_mode interface document
        - https://simon-ritchie.github.io/apysc/set_debug_mode.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> ap.set_debug_mode()
    >>> int_val: ap.Int = ap.Int(10)
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.DEBUG_MODE_SETTING.value
    query: str = f'DELETE FROM {table_name};'
    expression_data_util.exec_query(sql=query, commit=False)
    query = f'INSERT INTO {table_name}(is_debug_mode) VALUES(1);'
    expression_data_util.exec_query(sql=query)


def unset_debug_mode() -> None:
    """
    Unset the debug mode for the HTML and JavaScript debugging.

    References
    ----------
    - unset_debug_mode interface document
        - https://simon-ritchie.github.io/apysc/unset_debug_mode.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> ap.set_debug_mode()
    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.unset_debug_mode()
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.DEBUG_MODE_SETTING.value
    query: str = f'DELETE FROM {table_name};'
    expression_data_util.exec_query(sql=query)


def is_debug_mode() -> bool:
    """
    Get a boolean value whether the current debug mode is enabled or not.

    Returns
    -------
    result : bool
        If the current debug mode is enabled, this interface
        returns True.

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> ap.set_debug_mode()
    >>> ap.is_debug_mode()
    True

    >>> int_val: ap.Int = ap.Int(10)
    >>> ap.unset_debug_mode()
    >>> ap.is_debug_mode()
    False
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.DEBUG_MODE_SETTING.value
    query: str = f'SELECT is_debug_mode FROM {table_name} LIMIT 1;'
    expression_data_util.exec_query(sql=query)
    result_: Optional[Tuple[int]] = expression_data_util.cursor.fetchone()
    if result_ is None:
        return False
    result: bool = bool(result_[0])
    return result


def _get_callable_str(*, callable_: Union[Callable, str]) -> str:
    """
    Get a callable string (label).

    Parameters
    ----------
    callable_ : Callable or str
        Target function or method or property or dunder method name.

    Returns
    -------
    callable_str : str
        A callable string (label).
    """
    if isinstance(callable_, str):
        callable_str: str = callable_
    else:
        callable_str = callable_.__name__
    return callable_str


def _get_callable_path_name(
        *,
        callable_: Union[Callable, str],
        module_name: str,
        class_: Optional[Union[Type, str]] = None) -> str:
    """
    Get a specified callable count data path name.

    Parameters
    ----------
    callable_ : Callable or str
        Target function or method or property or dunder method name.
    module_name : str
        Module name. This value needs to set the `__name__` value.
    class_ : Type or str or None, optional
        Target class type or type name. If the target callable_
        variable is not a method, this interface ignores this argument.

    Returns
    -------
    path_name : str
        Target path name.
    """
    module_path: str = module_name.replace('.', '_')
    if class_ is None:
        class_name: str = ''
    elif inspect.isclass(class_):
        class_name_: str = getattr(class_, '__name__')
        class_name = f'_{class_name_}'
    else:
        class_name = f'_{class_}'
    callable_str: str = _get_callable_str(callable_=callable_)
    path_name: str = f'{module_path}{class_name}_{callable_str}'
    return path_name


def _get_callable_count(
        *,
        callable_: Union[Callable, str],
        module_name: str,
        class_: Optional[Union[Type, str]] = None) -> int:
    """
    Get a specified callable count number.

    Parameters
    ----------
    callable_ : Callable or str
        Target function or method or property or dunder method name.
    module_name : str
        Module name. This value needs to set the `__name__` value.
    class_ : Type or str or None, optional
        Target class type or type name. If the target callable_
        variable is not a method, this interface ignores this argument.

    Returns
    -------
    callable_count : int
        Target count number.
    """
    from apysc._expression import expression_data_util
    path_name: str = _get_callable_path_name(
        callable_=callable_, module_name=module_name, class_=class_)
    table_name: str = \
        expression_data_util.TableName.DEBUG_MODE_CALLABLE_COUNT.value
    query: str = (
        f'SELECT count FROM {table_name} '
        f"WHERE name = '{path_name}' LIMIT 1;"
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple[int]] = expression_data_util.cursor.fetchone()
    if result is None:
        return 0
    return result[0]


def _increment_callable_count(
        *,
        callable_: Union[Callable, str],
        module_name: str,
        class_: Optional[Union[Type, str]] = None) -> None:
    """
    Increment a specified callable count number.

    Parameters
    ----------
    callable_ : Callable or str
        Target function or method or property or dunder method name.
    module_name : str
        Module name. This value needs to set the `__name__` value.
    class_ : Type or str or None, optional
        Target class type or type name. If the target callable_
        variable is not a method, this interface ignores this argument.
    """
    from apysc._expression import expression_data_util
    callable_count: int = _get_callable_count(
        callable_=callable_, module_name=module_name, class_=class_)
    callable_count += 1
    path_name: str = _get_callable_path_name(
        callable_=callable_, module_name=module_name, class_=class_)
    table_name: str = \
        expression_data_util.TableName.DEBUG_MODE_CALLABLE_COUNT.value
    query: str = (
        f'DELETE FROM {table_name} '
        f"WHERE name = '{path_name}';"
    )
    expression_data_util.exec_query(sql=query, commit=False)
    query = (
        f'INSERT INTO {table_name}(name, count) '
        f"VALUES ('{path_name}', {callable_count});"
    )
    expression_data_util.exec_query(sql=query)


class DebugInfo:
    """
    Save debugging information (append callable interface
    name comment and arguments information) to the
    JavaScript expression file. The apysc uses this
    class at the `with` statement.

    Notes
    -----
    If the debug mode setting is not enabled, the apysc
    skips the saving.
    """

    _callable: Union[Callable, str]
    _module_name: str
    _args: List[Any]
    _kwargs: Dict[str, Any]
    _class_name: Optional[str]
    _DIVIDER: str = '/' * 70
    _callable_count: int
    _indent: Indent

    def __init__(
            self,
            *,
            callable_: Union[Callable, str],
            args: List[Any],
            kwargs: Dict[str, Any],
            module_name: str,
            class_name: Optional[str] = None) -> None:
        """
        Save debug information (append callable interface
        name comment and arguments information) to the
        JavaScript expression file. This class needs
        to use the `with` statement when instantiating.

        Notes
        -----
        If the debug mode setting is not enabled,
        this interface skips the saving.

        Parameters
        ----------
        callable_ : Callable or str
            Target function or method or property or dunder method name.
        args : list
            Function positional arguments.
        kwargs : dict
            Function keyword arguments.
        module_name : str
            Module name. This value requires the `__name__` value.
        class_name : str or None, optional
            Target class type name. If the target
            callable_ variable is not a method, this
            interface ignores this argument.
        """
        self._callable = callable_
        self._args = args
        self._kwargs = kwargs
        self._module_name = module_name
        self._class_name = class_name
        _increment_callable_count(
            callable_=callable_, module_name=module_name,
            class_=class_name)
        self._callable_count = _get_callable_count(
            callable_=callable_, module_name=module_name,
            class_=class_name)
        self._indent = Indent()

    def _get_class_info(self) -> str:
        """
        Get a class information string.

        Returns
        -------
        class_info : str
            Target class information string.
        """
        if self._class_name is None:
            class_info: str = ''
        else:
            class_info = f'\n// class: {self._class_name}'
        return class_info

    def __enter__(self) -> None:
        """
        This class uses this method at the start of the
        with-block.
        """
        import apysc as ap
        if not ap.is_debug_mode():
            return
        class_info: str = self._get_class_info()
        arguments_info: str = ''
        if self._args:
            arguments_info += (
                f'\n// Positional arguments: {self._args}'
            )
        if self._kwargs:
            arguments_info += f'\n// Keyword arguments: {self._kwargs}'
        callable_str: str = _get_callable_str(callable_=self._callable)
        expression: str = (
            f'{self._DIVIDER}'
            f'\n// [{callable_str} {self._callable_count}] '
            'started.'
            f'\n// module name: {self._module_name}'
            f'{class_info}'
            f'{arguments_info}'
        )
        ap.append_js_expression(expression=expression)
        self._indent.__enter__()

    def __exit__(self, *args: Any) -> None:
        """
        This class uses this method at the end of the with-block.

        Parameters
        ----------
        *args : list
            Positional arguments.
        """
        import apysc as ap
        if not ap.is_debug_mode():
            return
        class_info: str = self._get_class_info()
        callable_str: str = _get_callable_str(callable_=self._callable)
        expression: str = (
            f'// [{callable_str} {self._callable_count}] ended.'
            f'\n// module name: {self._module_name}'
            f'{class_info}'
            f'\n{self._DIVIDER}'
        )
        self._indent.__exit__()
        ap.append_js_expression(expression=expression)


# pyright: reportInvalidTypeVarUse=false
_F = TypeVar('_F', bound=Callable)


def add_debug_info_setting(
        module_name: str, *,
        class_name: Optional[str] = None) -> _F:
    """
    Set a debug information setting to a target
    callable object (decorator function).

    Parameters
    ----------
    module_name : str
        A target module name.
    class_name : str or None, default None
        Target class name. If a target callable is a function,
        this interface requires None of this argument.

    Returns
    -------
    wrapped : Callable
        Wrapped callable object.

    Notes
    -----
    Currently, this interface raises a mypy error under
    some mypy settings. Please set `type: ignore` comment
    if encountered its mypy error.

    Examples
    --------
    >>> import apysc as ap
    >>> @ap.add_debug_info_setting(  # type: ignore
    ...     module_name=__name__)
    ... def sample_method(a: int) -> None:
    ...     ...
    """

    def wrapped(func: _F) -> _F:
        """
        Wrapping function for a decorator setting.

        Parameters
        ----------
        func : Callable
            A target function or method to wrap.

        Returns
        -------
        inner_wrapped : Callable
            Wrapped callable object.
        """

        @functools.wraps(func)
        def inner_wrapped(*args: Any, **kwargs: Any) -> Any:
            """
            Wrapping function for a decorator setting.

            Parameters
            ----------
            *args : list
                Target positional arguments.
            **kwargs : dict
                Target keyword arguments.

            Returns
            -------
            result : Any
                A return value(s) of a callable execution result.
            """
            with DebugInfo(
                    callable_=func,
                    args=list(args),
                    kwargs=kwargs, module_name=module_name,
                    class_name=class_name):
                result: Any = func(*args, **kwargs)
                return result

        return inner_wrapped  # type: ignore

    return wrapped  # type: ignore



"""This module is for the SVG's `close path` (Z) path
data class implementation.
"""

from apysc._geom.path_data_base import PathDataBase
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String


class PathClose(PathDataBase):
    """
    Path data class for the SVG's `close path` (Z).

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color='#fff', thickness=3)
    >>> path: ap.Path = sprite.graphics.draw_path(
    ...     path_data_list=[
    ...         ap.PathMoveTo(x=0, y=00),
    ...         ap.PathLineTo(x=50, y=0),
    ...         ap.PathLineTo(x=50, y=50),
    ...         ap.PathClose(),
    ...     ])
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathClose')
    def __init__(self) -> None:
        """
        Path data class for the SVG's `close path` (Z).

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=3)
        >>> path: ap.Path = sprite.graphics.draw_path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=00),
        ...         ap.PathLineTo(x=50, y=0),
        ...         ap.PathLineTo(x=50, y=50),
        ...         ap.PathClose(),
        ...     ])
        """
        from apysc._geom.path_label import PathLabel
        super(PathClose, self).__init__(
            path_label=PathLabel.CLOSE,
            relative=False)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathClose')
    def _get_svg_str(self) -> str:
        """
        Get a path's SVG string created with the current setting.

        Returns
        -------
        svg_str : str
            An SVG path string was created with the current
            setting.
        """
        from apysc._type import value_util
        svg_char: String = self._get_svg_char()
        svg_str: str = value_util.get_value_str_for_expression(
            value=svg_char)
        return svg_str



"""This module is for the translation mapping data of the
following document:

Document file: graphics_fill_color.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics fill_color interface':
    '',

    'This page explains the `Graphics` class `fill_color` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `fill_color` property interface updates or get the instance\'s fill color.':  # noqa
    '',

    '## Basic usage':
    '',

    'The getter interface becomes the `String` hex color code value, and the setter one also requires the `String` hex color code value.\n\nThe following example changes the fill color (from cyan to magenta and magenta to cyan) when you click the rectangle:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    fill_color: ap.String = rectangle.fill_color\n    with ap.If(fill_color == \'#00aaff\'):\n        rectangle.fill_color = ap.String(\'#f0a\')\n    with ap.Else():\n        rectangle.fill_color = ap.String(\'#0af\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'./graphics_fill_color_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_fill_color_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## fill_color property API':
    '',

    '<!-- Docstring: apysc._display.fill_color_interface.FillColorInterface.fill_color -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get this instance\'s fill color.<hr>\n\n**[Returns]**\n\n- `fill_color`: String\n  - Current fill color (hexadecimal string, e.g., \'#00aaff\'). If not be set, this interface returns a blank string.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.fill_color = ap.String(\'#f0a\')\n>>> rectangle.fill_color\nString(\'#ff00aa\')\n```':  # noqa
    '',

}



"""This module is for interfaces and definitions for
a last expression's scope information.
"""

from enum import Enum
from typing import Optional
from typing import Tuple


class LastScope(Enum):

    NORMAL = 1
    IF = 2
    ELIF = 3
    ELSE = 4
    FOR = 5
    EVENT_HANDLER = 6


def reset() -> None:
    """
    Reset last expression's scope information.
    """
    from apysc._expression import expression_data_util
    query: str = (
        f'DELETE FROM {expression_data_util.TableName.LAST_SCOPE.value};'
    )
    expression_data_util.exec_query(sql=query)


def get_last_scope() -> LastScope:
    """
    Get last scope value.

    Returns
    -------
    last_scope : LastScope
        Last scope value. If there is no last scope's
        value, this interface returns LastScope.NORMAL.
    """
    from apysc._expression import expression_data_util
    query: str = (
        'SELECT last_scope FROM '
        f'{expression_data_util.TableName.LAST_SCOPE.value} '
        'LIMIT 1;'
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple[int]] = expression_data_util.cursor.fetchone()
    if result is None:
        return LastScope.NORMAL
    last_scope: LastScope = LastScope(result[0])
    return last_scope


def set_last_scope(*, value: LastScope) -> None:
    """
    Set last scope value.

    Parameters
    ----------
    value : LastScope
        Last scope value to set.
    """
    from apysc._expression import expression_data_util
    reset()
    query: str = (
        'INSERT INTO '
        f'{expression_data_util.TableName.LAST_SCOPE.value}(last_scope) '
        f'VALUES({value.value});'
    )
    expression_data_util.exec_query(sql=query)



"""This module is for the Japanese fixed-translation mappings
settings.
"""

from apysc._lint_and_doc.fixed_translation_mapping.data_model import Mapping
from apysc._lint_and_doc.fixed_translation_mapping.data_model import Mappings

MAPPINGS: Mappings = Mappings(
    mappings=[
        Mapping(
            key='## See also',
            value='## 関連資料'),
        Mapping(
            key='**[Parameters]**',
            value='**[引数]**'),
        Mapping(
            key='**[Returns]**',
            value='**[返却値]**'),
        Mapping(
            key='**[Parameters]**',
            value='**[引数]**'),
        Mapping(
            key='**[Examples]**',
            value='**[コードサンプル]**'),
        Mapping(
            key='**[References]**',
            value='**[関連資料]**'),
        Mapping(
            key=(
                '<span class="inconspicuous-txt">Note: the document build '
                'script generates and updates this API document section '
                'automatically. Maybe this section is duplicated '
                'compared with previous sections.</span>'
            ),
            value=(
                '<span class="inconspicuous-txt">特記事項: このAPI'
                'ドキュメントはドキュメントビルド用のスクリプトによって自動で'
                '生成・同期されています。そのためもしかしたらこの節の'
                '内容は前節までの内容と重複している場合があります。</span>'
            )),
    ])



"""Class implementation for the rotation around the center
point animation value.
"""

from typing import Generic
from typing import TypeVar
from typing import Union

from apysc._animation.animation_base import AnimationBase
from apysc._animation.easing import Easing
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_interface import VariableNameInterface

_T = TypeVar('_T', bound=VariableNameInterface)


class AnimationRotationAroundCenter(AnimationBase[_T], Generic[_T]):
    """
    The animation class for a rotation around the center point.

    References
    ----------
    - animation_rotation_around_center interface document
        - https://simon-ritchie.github.io/apysc/animation_rotation_around_center.html  # noqa
    - Animation interfaces duration setting document
        - https://simon-ritchie.github.io/apysc/animation_duration.html
    - Animation interfaces delay setting document
        - https://simon-ritchie.github.io/apysc/animation_delay.html
    - Each animation interface return value document
        - https://simon-ritchie.github.io/apysc/animation_return_value.html  # noqa
    - Sequential animation setting document
        - https://simon-ritchie.github.io/apysc/sequential_animation.html
    - animation_parallel interface document
        - https://simon-ritchie.github.io/apysc/animation_parallel.html
    - Easing enum document
        - https://simon-ritchie.github.io/apysc/easing_enum.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> animation: ap.AnimationRotationAroundCenter
    >>> animation = rectangle.animation_rotation_around_center(
    ...     rotation_around_center=90,
    ...     duration=1500,
    ...     easing=ap.Easing.EASE_OUT_QUINT,
    ... )
    >>> _ = animation.start()
    """

    _rotation_around_center: Int
    _before_rotation_around_center: Int
    _rotation_around_center_diff: Int

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationRotationAroundCenter')
    def __init__(
            self,
            *,
            target: _T,
            rotation_around_center: Union[int, Int],
            duration: Union[int, Int] = 3000,
            delay: Union[int, Int] = 0,
            easing: Easing = Easing.LINEAR) -> None:
        """
        The animation class for a rotation around the center point.

        Parameters
        ----------
        target : SkewXInterface
            A target instance of the animation target
            (e.g., `Rectangle` instance).
        rotation_around_center : Int or int
            The final rotation around the center point of the animation.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Raises
        ------
        TypeError
            If a specified target is not a RotationAroundCenterInterface
            instance.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._display.rotation_around_center_interface import \
            RotationAroundCenterInterface
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        variable_name: str = expression_variables_util.\
            get_next_variable_name(
                type_name=var_names.ANIMATION_ROTATION_AROUND_CENTER)
        target_: VariableNameInterface = target
        if isinstance(target_, RotationAroundCenterInterface):
            target_.\
                _initialize_rotation_around_center_if_not_initialized()
            self._before_rotation_around_center = \
                target_._rotation_around_center
        else:
            raise TypeError(
                'Specified `target` argument is not a '
                'RotationAroundCenterInterface instance: '
                f'{type(target_)}')
        self._rotation_around_center = to_apysc_val_from_builtin.\
            get_copied_int_from_builtin_val(
                integer=rotation_around_center)
        self._rotation_around_center_diff = self._rotation_around_center \
            - self._before_rotation_around_center
        self._set_basic_animation_settings(
            target=target,
            duration=duration,
            delay=delay,
            easing=easing)
        super(AnimationRotationAroundCenter, self).__init__(
            variable_name=variable_name)

    def _get_animation_func_expression(self) -> str:
        """
        Get a animation function expression.

        Returns
        -------
        expression : str
            Animation function expression.
        """
        from apysc._type import value_util
        diff_str: str = value_util.get_value_str_for_expression(
            value=self._rotation_around_center_diff)
        return f'\n  .rotate({diff_str});'

    def _get_complete_event_in_handler_head_expression(self) -> str:
        """
        Get an expression to be inserted into the complete event
        handler's head.

        Returns
        -------
        expression : str
            An expression to insert into the complete event
            handler's head.
        """
        from apysc._display.rotation_around_center_interface import \
            RotationAroundCenterInterface
        expression: str = ''
        if isinstance(self._target, RotationAroundCenterInterface):
            self._target.\
                _initialize_rotation_around_center_if_not_initialized()
            expression = (
                f'{self._target._rotation_around_center.variable_name} = '
                f'{self._rotation_around_center.variable_name};'
            )
        return expression



"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_comparison_operations.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Int and Number basic comparison operations':
    '',

    'This page explains basic comparison operations of the `Int` and `Number` classes, like the `>=`\\, `<`\\.':  # noqa
    '',

    '## Common behaviors':
    '',

    'Each comparison operation returns a `Boolean` value, not a Python built-in `bool` value:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert isinstance(result, ap.Boolean)\n```':  # noqa
    '',

    'You can compare the `Int` or `Number` values with the Python built-in values, like the `int` or `float`:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(20)\nresult: ap.Boolean = int_1 == 20\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == 10.5\nassert result\n```':  # noqa
    '',

    'Also, the comparison between the `Int` and `Int`\\, `Number` and `Number`\\, `Int` and `Number` are supported:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == int_2\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = ap.Number(10.5)\nresult: ap.Boolean = number_1 == number_2\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = ap.Number(10)\nresult: ap.Boolean = int_1 == number_1\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Equal comparison operator':
    '',

    'You can use the `==` operator for the equal comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 == 10\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Not equal comparison operator':
    '',

    'You can use the `!=` operator for the not equal comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 != 15\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Less than comparison operator':
    '',

    'You can use the `<` operator for the less than comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 < 11\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Less than or equal comparison operator':
    '',

    'You can use the `<=` operator for the less than or equal comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 <= 10\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Greater than comparison operator':
    '',

    'You can use the `>` operator for the greater than comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 > 9\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Greater than or equal comparison operator':
    '',

    'You can use the `>=` operator for the greater than or equal comparison:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nresult: ap.Boolean = int_1 >= 10\nassert result\n```':  # noqa
    '',

}



"""This module is for the `return` expression class
implementation.
"""


class Return:
    """
    Class for the return expression.

    References
    ----------
    - Return document
        - https://simon-ritchie.github.io/apysc/return.html
    """

    def __init__(self) -> None:
        """
        Class for the return expression.

        Notes
        -----
        This class can be instantiated only in an event handler scope.

        References
        ----------
        - Return document
            - https://simon-ritchie.github.io/apysc/return.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_timer(e: ap.TimerEvent, options: dict) -> None:
        ...     \"\"\"
        ...     The handler that the timer calls.
        ...
        ...     Parameters
        ...     ----------
        ...     e : ap.TimerEvent
        ...         Event instance.
        ...     options : dict
        ...         Optional arguments dictionary.
        ...     \"\"\"
        ...     with ap.If(e.this.current_count > 10):
        ...         ap.Return()
        ...     ap.trace('Not returned.')
        >>> ap.Timer(on_timer, delay=100).start()
        """
        import apysc as ap
        self._validate_current_scope_is_event_handler()
        ap.append_js_expression(expression='return;')

    def _validate_current_scope_is_event_handler(self) -> None:
        """
        Validate whether the current scope is an event handler
        scope or not.
        """
        from apysc._expression import event_handler_scope
        event_handler_scope_count: int = \
            event_handler_scope.get_current_event_handler_scope_count()
        if event_handler_scope_count > 0:
            return
        raise Exception(
            'The `Return` class can be instantiated only in an event '
            'handler scope.')
